<?php


namespace Intergo\Log\Contracts;


use ReflectionClass;

abstract class AbstractEnumContract implements EnumContract
{
    public static array $mapping_fields = [];

    public static array $integer_fields = [];

    public static array $float_fields = [];

    public static array $string_fields = [];

    /**
     * This method will check if the field exist as a constant above.
     * @param $field
     * @return bool
     */
    public function doesFieldExist($field): bool
    {
        $oClass = new ReflectionClass(__CLASS__);
        $constants = array_values($oClass->getConstants());
        if (in_array($field, $constants)) {
            return true;
        }
        return false;
    }

    /**
     * Use this method to get the correct typeCast value so we keep the consistency in Graylog
     * @param $data
     * @return array
     */
    public function typeCastValues($data): array
    {
        $final_data = [];
        foreach ($data as $field => $value) {
            $field = $this->getMappingField($field);
            if (in_array($field, static::$integer_fields)) {
                $final_data[$field] = intval($value);
            } elseif (in_array($field, static::$float_fields)) {
                $final_data[$field] = floatval($value);
            } elseif (in_array($field, static::$string_fields)) {
                $final_data[$field] = $value;
            } elseif (is_array($value)) {
                $final_data[$field] = json_encode($value);
            } else {
                $is_value_json_decoded = @json_decode($value);

                if (is_null($is_value_json_decoded)) {
                    if (!isset($final_data['extra_params'])) {
                        $final_data['extra_params'] = "";
                    }
                    $final_data['extra_params'] .= json_encode([$field => $value]);
                }
            }
        }
        return $final_data;
    }

    /**
     * Use this method to get the relevant mapping field based on the different variations we have on code
     * @param $field
     * @return string
     */
    public function getMappingField($field): string
    {
        return static::$mapping_fields[$field] ?? $field;
    }
}
